#!/usr/local/bin/python2.4

import re;

def getAvg(numList):
    return reduce(lambda x,y: (x+y)/2, numList);

def getMapCalibration(mapDataFile):
    fcon = open(mapDataFile).readlines()
    calPointsPixels = []
    calPointsCoord = []
    for i in fcon:
        if (i.startswith('MMPLL')):
            calPointsCoord.append(i.strip())
        elif (i.startswith('MMPXY')):
            calPointsPixels.append(i.strip())
    pointPixels = {}
    for i in calPointsPixels:
        #print i.split(',')
        (mmpxy, pointId, xPixel, yPixel) = i.split(',')
        pointPixels[int(pointId)] = [int(xPixel), int(yPixel)]
    pointCoord = {}
    for i in calPointsCoord:
        (mmpll, pointId, xCoord, yCoord) = i.split(',')
        pointCoord[int(pointId)] = [float(xCoord), float(yCoord)]

    ret = {}
    for i in pointPixels.keys():
        ret[i] = {}
        ret[i]['pos'] = pointPixels[i]
        ret[i]['coord'] = pointCoord[i]

    return ret

def avg(l):
    return sum(l) / float(len(l))

# 0 for x axis, 1 for y
class AxisInfo:
    def __init__(self, cal_data, i):
        self._cal_data = cal_data
        self._axis = i
        self.calcPixelOnMap()

    def getAxisCoords(self):
        mCal = self._cal_data
        i = self._axis
        return [(mCal[t]['pos'][i], mCal[t]['coord'][i]) for t in mCal.keys()]

    def calcPixelOnMap(self):
        a_coord = self.getAxisCoords()
        map_pixels = [t[0] for t in a_coord]
        mid = (min(map_pixels) + max(map_pixels)) / float(2)
        lower_coords = [t for t in a_coord if t[0] < mid]
        higher_coords = [t for t in a_coord if t[0] >= mid]

        # Sanity
        if (len(set([t[0] for t in lower_coords])) * 2) + (len(set([t[0] for t in higher_coords])) * 2) < len(map_pixels):
            print 'Rare case encountered. Pixels on this axis are:',map_pixels

        low_pixel = avg([t[0] for t in lower_coords])
        low_pixel_coord = avg([t[1] for t in lower_coords])
        high_pixel = avg([t[0] for t in higher_coords])
        high_pixel_coord = avg([t[1] for t in higher_coords])
        """
        if (high_pixel_coord < low_pixel_coord):
            print 'Axis:',self._axis,'Changing mult to -1:',
            self._mult = -1
            print self._mult
        """

        self._middle_pixel = (low_pixel + high_pixel) / float(2)
        degs_diff = (high_pixel_coord - low_pixel_coord)
        pixels_diff = (high_pixel - low_pixel)
        self._degs_per_pixel = degs_diff / float(pixels_diff)
        self._pixels_per_deg = (pixels_diff) / float(abs(degs_diff))
        self._start_deg = low_pixel_coord

    def getAxisMiddle(self):
        ret = (self._middle_pixel, self._degs_per_pixel * self._middle_pixel + self._start_deg)
        return ret

    def pixelToDegs(self, p):
        ret = (self._degs_per_pixel * p + self._start_deg)
        #print 'Axis',self._axis,'Returns',ret,'(DpP = ',self._degs_per_pixel,')'
        return ret

    def pixels_per_deg(self):
        return self._pixels_per_deg


class MapCalibration:
    def __init__(self, mapDataFn):
        self._cal = getMapCalibration(mapDataFn)
        self._x = AxisInfo(self._cal, 0)
        self._y = AxisInfo(self._cal, 1)

    def pixelsToDegs(self, p_x, p_y):
        return (self._x.pixelToDegs(p_x), self._y.pixelToDegs(p_y))

    def pixels_per_deg(self):
        dx = self._x.pixels_per_deg()
        dy = self._y.pixels_per_deg()
        return (dx + dy)
