#!/usr/bin/env python

import PIL
from PIL import Image
from calc_split import *
from mapcaldata import *
import os
import sys
from sys import argv

class MapOutputData:
    def __init__(self, in_dir, out_dir):
        self._idx = 1
        self._out_dir = out_dir
        self._in_dir = in_dir
        mk_fn = os.path.join(out_dir, 'map_koord.txt')
        self._map_koord = open(mk_fn, 'w')

    def done(self):
        self._map_koord.close()

    def map_image(self, map_fn):
        return os.path.join(self._in_dir, map_fn + '.jpg')

    def map_cal(self, map_fn):
        return os.path.join(self._in_dir, map_fn + '.map')

    def next_idx(self):
        self._idx += 1
        return self._idx - 1

    def koord_write(self, ln):
        self._map_koord.write(ln + '\n')

def split_single_map(base_fn, mout):
    im = Image.open(mout.map_image(base_fn))
    mcal = MapCalibration(mout.map_cal(base_fn))

    s1 = calc_splits(im.getbbox()[2],1280)
    s2 = calc_splits(im.getbbox()[3],1024)
    for i in s1:
        for j in s2:
            out_map_fn = 'map_file%.4d.jpg' % mout.next_idx()
            print 'File:',out_map_fn,
            print 'Cutting from pixels:',i[0],j[0],i[1],j[1],' middle:',
            middle_x = (i[0] + i[1]) / float(2)
            middle_y = (j[0] + j[1]) / float(2)
            print mcal.pixelsToDegs(middle_x, middle_y)
            d_x, d_y = mcal.pixelsToDegs(middle_x, middle_y)

            ti = im.crop((i[0],j[0],i[1],j[1]))
            ti.load()
            ti.save(os.path.join(mout._out_dir, out_map_fn))
            mout.koord_write('%s %.5f %.5f %d' % (out_map_fn, d_y, d_x, mcal.pixels_per_deg()))

if __name__ == '__main__':
    if len(argv) < 3:
        print 'Usage:',argv[0],'<in dir> <out dir>'
        print 'Where in_dir contains oziexplorer files.'
        sys.exit(1)
    mout = MapOutputData(argv[1], argv[2])
    all_ozi_files = [t for t in os.listdir(argv[1]) if t.endswith('.map')]
    for f in all_ozi_files:
        print '########## Now processing',f,'##########'
        base_fn = f.split('.')[0]
        split_single_map(base_fn, mout)

    mout.done()
